@@ -102,6 +102,12 @@ struct _GeditPreferencesDialog
 	
 	/* Line numbers */
 	GtkWidget	*display_line_numbers_checkbutton;
+
+	/* Highlight current line */
+	GtkWidget	*highlight_current_line_checkbutton;
+	
+	/* Highlight matching bracket */
+	GtkWidget	*bracket_matching_checkbutton;
 	
 	/* Right margin */
 	GtkWidget	*right_margin_checkbutton;
@@ -411,6 +417,27 @@ display_line_numbers_checkbutton_toggled (GtkToggleButton        *button,
 	gedit_prefs_manager_set_display_line_numbers (gtk_toggle_button_get_active (button));
 }
 
+static void
+highlight_current_line_checkbutton_toggled (GtkToggleButton        *button,
+					  GeditPreferencesDialog *dlg)
+{
+	g_return_if_fail (button == 
+			GTK_TOGGLE_BUTTON (dlg->highlight_current_line_checkbutton));
+
+	gedit_prefs_manager_set_highlight_current_line (gtk_toggle_button_get_active (button));
+}
+
+static void
+bracket_matching_checkbutton_toggled (GtkToggleButton        *button,
+				      GeditPreferencesDialog *dlg)
+{
+	g_return_if_fail (button == 
+			GTK_TOGGLE_BUTTON (dlg->bracket_matching_checkbutton));
+
+	gedit_prefs_manager_set_bracket_matching (
+				gtk_toggle_button_get_active (button));
+}
+
 static gboolean split_button_state = TRUE;
 
 static void
@@ -494,6 +521,12 @@ setup_view_page (GeditPreferencesDialog *dlg)
 	/* Set initial state */
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dlg->display_line_numbers_checkbutton),
 				gedit_prefs_manager_get_display_line_numbers ());
+	
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dlg->highlight_current_line_checkbutton),
+				gedit_prefs_manager_get_highlight_current_line ());
+	
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dlg->bracket_matching_checkbutton),
+				      gedit_prefs_manager_get_bracket_matching ());
 
 	wrap_mode = gedit_prefs_manager_get_wrap_mode ();
 	switch (wrap_mode )
@@ -534,6 +567,12 @@ setup_view_page (GeditPreferencesDialog *dlg)
 	gtk_widget_set_sensitive (dlg->display_line_numbers_checkbutton,
 				  gedit_prefs_manager_display_line_numbers_can_set ());
 
+	gtk_widget_set_sensitive (dlg->highlight_current_line_checkbutton,
+				  gedit_prefs_manager_highlight_current_line_can_set ());
+
+	gtk_widget_set_sensitive (dlg->bracket_matching_checkbutton,
+				  gedit_prefs_manager_bracket_matching_can_set ());
+
 	wrap_mode_can_set = gedit_prefs_manager_wrap_mode_can_set ();
 
 	gtk_widget_set_sensitive (dlg->wrap_text_checkbutton, 
@@ -554,6 +593,12 @@ setup_view_page (GeditPreferencesDialog *dlg)
 	g_signal_connect (G_OBJECT (dlg->display_line_numbers_checkbutton), "toggled", 
 			  G_CALLBACK (display_line_numbers_checkbutton_toggled), 
 			  dlg);
+	g_signal_connect (G_OBJECT (dlg->highlight_current_line_checkbutton), "toggled", 
+			  G_CALLBACK (highlight_current_line_checkbutton_toggled), 
+			  dlg);
+	g_signal_connect (G_OBJECT (dlg->bracket_matching_checkbutton), "toggled", 
+			  G_CALLBACK (bracket_matching_checkbutton_toggled), 
+			  dlg);
 	g_signal_connect (G_OBJECT (dlg->wrap_text_checkbutton), "toggled", 
 			  G_CALLBACK (wrap_mode_checkbutton_toggled), 
 			  dlg);
@@ -797,7 +842,7 @@ language_changed_cb (GtkOptionMenu *optionmenu,
 	GtkTreeIter iter;
 	GtkTreePath *path;
 
-	languages = gtk_source_languages_manager_get_available_languages (
+	languages = gedit_languages_manager_get_available_languages_sorted (
 						gedit_get_languages_manager ());
 
 	lang = g_slist_nth_data ((GSList*)languages, gtk_option_menu_get_history (optionmenu));
@@ -841,7 +886,7 @@ get_selected_language (GeditPreferencesDialog *dlg)
 	const GSList *languages;
 	GtkSourceLanguage *lang;
 	
-	languages = gtk_source_languages_manager_get_available_languages (
+	languages = gedit_languages_manager_get_available_languages_sorted (
 						gedit_get_languages_manager ());
 	lang = g_slist_nth_data ((GSList*)languages,
 		gtk_option_menu_get_history (GTK_OPTION_MENU (dlg->hl_mode_optionmenu)));
@@ -1122,6 +1167,9 @@ setup_syntax_highlighting_page (GeditPreferencesDialog *dlg)
 	const GSList *languages, *l;
 	GtkCellRenderer *renderer;
 	GtkTreeViewColumn *column;
+	GeditDocument *current_document;
+	GtkSourceLanguage *current_document_language;
+	gint selected_language, current_item;
 
 	/* Set initial state */
 	hl_enabled = gedit_prefs_manager_get_enable_syntax_highlighting ();
@@ -1156,11 +1204,21 @@ setup_syntax_highlighting_page (GeditPreferencesDialog *dlg)
 	/* Add languages to optionmenu. */
 	menu = gtk_menu_new ();
 
-	languages = gtk_source_languages_manager_get_available_languages (
+	languages = gedit_languages_manager_get_available_languages_sorted (
 						gedit_get_languages_manager ());
 
 	l = languages;
 
+	current_document = gedit_get_active_document ();
+	if (current_document != NULL) 
+		current_document_language = gedit_document_get_language (
+						current_document);
+	else 
+		current_document_language = NULL;
+
+	selected_language = 0;
+	current_item = 0;
+	
 	while (l != NULL)
 	{
 		GtkSourceLanguage *lang = GTK_SOURCE_LANGUAGE (l->data);
@@ -1175,12 +1233,19 @@ setup_syntax_highlighting_page (GeditPreferencesDialog *dlg)
 		
 		gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
 
+		if (current_document_language == lang)
+			selected_language = current_item;
+
+		++current_item;
+		
 		l = g_slist_next (l);
 	}
 	
 	gtk_widget_show_all (menu);
 	
 	gtk_option_menu_set_menu (GTK_OPTION_MENU (dlg->hl_mode_optionmenu), menu);
+	gtk_option_menu_set_history (GTK_OPTION_MENU (dlg->hl_mode_optionmenu), 
+				     selected_language);
 
 	g_signal_connect (G_OBJECT (dlg->enable_syntax_hl_checkbutton), "toggled",
 			  G_CALLBACK (enable_syntax_hl_button_toggled), dlg);
@@ -1214,7 +1279,11 @@ setup_plugins_page (GeditPreferencesDialog *dlg)
 	page_content = gedit_plugin_manager_get_page ();
 	g_return_if_fail (page_content != NULL);
 	
-	gtk_box_pack_start (GTK_BOX (dlg->plugin_manager_place_holder), page_content, TRUE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (dlg->plugin_manager_place_holder), 
+			    page_content, 
+			    TRUE, 
+			    TRUE, 
+			    0);
 
 }
 
@@ -1249,6 +1318,10 @@ get_preferences_dialog (GtkWindow *parent)
 
 	dialog->dialog = glade_xml_get_widget (gui, "preferences_dialog");
 	dialog->display_line_numbers_checkbutton = glade_xml_get_widget (gui, "display_line_numbers_checkbutton");
+	dialog->highlight_current_line_checkbutton = glade_xml_get_widget (gui, "highlight_current_line_checkbutton");
+
+	dialog->bracket_matching_checkbutton = glade_xml_get_widget (gui, "bracket_matching_checkbutton");
+
 	dialog->wrap_text_checkbutton = glade_xml_get_widget (gui, "wrap_text_checkbutton");
 	dialog->split_checkbutton = glade_xml_get_widget (gui, "split_checkbutton");
 
@@ -1303,6 +1376,8 @@ get_preferences_dialog (GtkWindow *parent)
 
 	if (!dialog->dialog ||
 	    !dialog->display_line_numbers_checkbutton ||
+	    !dialog->highlight_current_line_checkbutton ||
+	    !dialog->bracket_matching_checkbutton ||
 	    !dialog->wrap_text_checkbutton ||
 	    !dialog->split_checkbutton ||
 	    !dialog->default_font_checkbutton ||
