@@ -293,26 +293,17 @@ gedit_utils_is_uri_read_only (const gchar* uri)
 	return res;	
 }
 
-/* lifted from eel */
 static GtkWidget *
 gedit_gtk_button_new_with_stock_icon (const gchar *label, const gchar *stock_id)
 {
-	GtkWidget *button, *l, *image, *hbox, *align;
+	GtkWidget *button;
 
-	/* This is mainly copied from gtk_button_construct_child(). */
-	button = gtk_button_new ();
-	l = gtk_label_new_with_mnemonic (label);
-	gtk_label_set_mnemonic_widget (GTK_LABEL (l), GTK_WIDGET (button));
-	image = gtk_image_new_from_stock (stock_id, GTK_ICON_SIZE_BUTTON);
-	hbox = gtk_hbox_new (FALSE, 2);
-	align = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
-	gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, FALSE, 0);
-	gtk_box_pack_end (GTK_BOX (hbox), l, FALSE, FALSE, 0);
-	gtk_container_add (GTK_CONTAINER (button), align);
-	gtk_container_add (GTK_CONTAINER (align), hbox);
-	gtk_widget_show_all (align);
+	button = gtk_button_new_with_mnemonic (label);
+	gtk_button_set_image (GTK_BUTTON (button),
+			      gtk_image_new_from_stock (stock_id,
+							GTK_ICON_SIZE_BUTTON));
 
-	return button;
+        return button;
 }
 
 GtkWidget*
@@ -598,7 +589,7 @@ gedit_warning (GtkWindow *parent, gchar *format, ...)
 			GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
 		   	GTK_MESSAGE_ERROR,
 		   	GTK_BUTTONS_OK,
-			str);
+			"%s", str);
 
 	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
 
@@ -615,49 +606,49 @@ gedit_warning (GtkWindow *parent, gchar *format, ...)
 /* the following functions are taken from eel */
 
 gchar *
-gedit_utils_str_middle_truncate (const char *string,
-				 guint truncate_length)
+gedit_utils_str_middle_truncate (const gchar *string,
+				 guint        truncate_length)
 {
-	char *truncated;
-	guint length;
-	guint num_left_chars;
-	guint num_right_chars;
+	GString     *truncated;
+	guint        length;
+	guint        n_chars;
+	guint        num_left_chars;
+	guint        right_offset;
+	guint        delimiter_length;
+	const gchar *delimiter = "\342\200\246";
 
-	/* we use "…" instead of "..." */
-	const char delimter[] = "…";
-	const guint delimter_length = strlen (delimter);
-	const guint min_truncate_length = delimter_length + 2;
+	g_return_val_if_fail (string != NULL, NULL);
 
-	if (string == NULL) {
-		return NULL;
-	}
+	length = strlen (string);
+
+	g_return_val_if_fail (g_utf8_validate (string, length, NULL), NULL);
 
 	/* It doesnt make sense to truncate strings to less than
 	 * the size of the delimiter plus 2 characters (one on each
 	 * side)
 	 */
-	if (truncate_length < min_truncate_length) {
+	delimiter_length = g_utf8_strlen (delimiter, -1);
+	if (truncate_length < (delimiter_length + 2)) {
 		return g_strdup (string);
 	}
 
-	length = strlen (string);
+	n_chars = g_utf8_strlen (string, length);
 
 	/* Make sure the string is not already small enough. */
-	if (length <= truncate_length) {
+	if (n_chars <= truncate_length) {
 		return g_strdup (string);
 	}
 
 	/* Find the 'middle' where the truncation will occur. */
-	num_left_chars = (truncate_length - delimter_length) / 2;
-	num_right_chars = truncate_length - num_left_chars - delimter_length + 1;
+	num_left_chars = (truncate_length - delimiter_length) / 2;
+	right_offset = n_chars - truncate_length + num_left_chars + delimiter_length;
 
-	truncated = g_new (char, truncate_length + 1);
-
-	strncpy (truncated, string, num_left_chars);
-	strncpy (truncated + num_left_chars, delimter, delimter_length);
-	strncpy (truncated + num_left_chars + delimter_length, string + length - num_right_chars + 1, num_right_chars);
-	
-	return truncated;
+	truncated = g_string_new_len (string,
+				      g_utf8_offset_to_pointer (string, num_left_chars) - string);
+	g_string_append (truncated, delimiter);
+	g_string_append (truncated, g_utf8_offset_to_pointer (string, right_offset));
+		
+	return g_string_free (truncated, FALSE);
 }
 
 gchar *
